# Copyright (C) 2002-2003 by James Henstridge <james@daa.com.au>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, US

"""Perform spam detection with SpamAssassin.

Messages are passed to a spamd (SpamAssassin) daemon for spam checking.
Depending on the score returned, messages may be rejected or held for
moderation.
"""

import string
import spamd

from Mailman import mm_cfg
from Mailman import Errors
from Mailman.Logging.Syslog import syslog
from Mailman.Handlers import Hold
from Mailman.Handlers.Moderate import matches_p

SPAMD_HOST    = getattr(mm_cfg, 'SPAMASSASSIN_HOST', '')
DISCARD_SCORE = getattr(mm_cfg, 'SPAMASSASSIN_DISCARD_SCORE', 10)
HOLD_SCORE    = getattr(mm_cfg, 'SPAMASSASSIN_HOLD_SCORE', 5)
MEMBER_BONUS  = getattr(mm_cfg, 'SPAMASSASSIN_MEMBER_BONUS', 2)

class SpamAssassinDiscard(Errors.DiscardMessage):
    '''The message was scored above the discard threshold'''
    reason = 'SpamAssassin identified this message as spam'
    rejection = 'Your message has been discarded as spam'

class SpamAssassinHold(Errors.HoldMessage):
    '''The message was scored above the hold threshold'''
    def __init__(self, score=-1, symbols=''):
        Errors.HoldMessage.__init__(self)
        self.reason = 'SpamAssassin identified this message as possible ' \
                      'spam (score %g)' % score
        self.rejection = 'Your message was held for moderation because ' \
                         'SpamAssassin gave the message a score of %g ' \
                         'for the following reasons:\n\n%s' % \
                         (score, symbols)

def check_message(mlist, message):
    '''Check a message against a SpamAssassin spamd process.
    Returns a tuple of the form (score, symbols)'''
    try:
        connection = spamd.SpamdConnection(SPAMD_HOST)
        # identify as the mailing list, to allow storing per-list
        # AWL and bayes databases.
        connection.addheader('User', mlist.internal_name())
        res = connection.check(spamd.SYMBOLS, message)

        score = connection.getspamstatus()[1]
        symbols = connection.response_message.replace(',', ', ')

        return score, symbols
    except spamd.error, ex:
        syslog('error', 'spamd: %s' % str(ex))
        return -1, ''

def process(mlist, msg, msgdata):
    if msgdata.get('approved'):
        return
    
    score, symbols = check_message(mlist, str(msg))

    if MEMBER_BONUS != 0:
        for sender in msg.get_senders():
            if mlist.isMember(sender) or \
                   matches_p(sender, mlist.accept_these_nonmembers, mlist.internal_name()):
                score -= MEMBER_BONUS
                break

    if score > DISCARD_SCORE:
        listname = mlist.real_name
        sender = msg.get_sender()
        syslog('vette', '%s post from %s discarded: '
                        'SpamAssassin score was %g (discard threshold is %g)'
                          % (listname, sender, score, DISCARD_SCORE))
        raise SpamAssassinDiscard
    elif score > HOLD_SCORE:
        Hold.hold_for_approval(mlist, msg, msgdata,
                               SpamAssassinHold(score, symbols))



#!/usr/bin/python
#
#    test-mailman.py quality assurance test script for mailman
#    Copyright (C) 2010 Canonical Ltd.
#    Author: Marc Deslauriers <marc.deslauriers@canonical.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License version 3,
#    as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# packages required for test to run:
# QRT-Packages: apache2-mpm-prefork elinks postfix mailman procmail
# packages where more than one package can satisfy a runtime requirement:
# QRT-Alternates: 
# files and directories required for the test to run:
# QRT-Depends: testlib_httpd.py 
# privilege required for the test to run (remove line if running as user is okay):
# QRT-Privilege: root
# QRT-Conflicts: apache2-mpm-event apache2-mpm-itk apache2-mpm-worker exim4

'''
    In general, this test should be run in a virtual machine (VM) or possibly
    a chroot and not on a production machine. While efforts are made to make
    these tests non-destructive, there is no guarantee this script will not
    alter the machine. You have been warned.

    When installing mailman, be sure to install the english language files
    When postfix installs, select "local only"

    How to run in a clean VM:
    $ sudo apt-get -y install python-unit <QRT-Packages> && sudo ./test-mailman.py -v'

    How to run in a clean schroot named 'lucid':
    $ schroot -c lucid -u root -- sh -c 'apt-get -y install python-unit <QRT-Packages> && ./test-mailman.py -v'
'''


import unittest, subprocess, sys, os, time, smtplib
import urllib, urllib2, cookielib, re, tempfile
import testlib
import testlib_httpd

try:
    from private.qrt.mailman import PrivateMailmanTest
except ImportError:
    class PrivateMailmanTest(object):
        '''Empty class'''
    print >>sys.stdout, "Skipping private tests"

class MailmanTest(testlib_httpd.HttpdCommon, PrivateMailmanTest):
    '''Test Mailman.'''

    def setUp(self):
        '''Set up prior to each test_* function'''
        self.mailman_daemon = testlib.TestDaemon("/etc/init.d/mailman")
        self.mailman_cfg = '/etc/mailman/mm_cfg.py'
        self.mailman_aliases = '/var/lib/mailman/data/aliases'
        self.mailman_pid = '/var/run/mailman/mailman.pid'
        self.postfix_daemon = testlib.TestDaemon("/etc/init.d/postfix")
        self.postfix_mastercf = '/etc/postfix/master.cf'
        self.postfix_maincf = '/etc/postfix/main.cf'
        self.postfix_transport = '/etc/postfix/transportqrt'
        self.postfix_aliases = '/etc/aliases'
        self.apache_pid = "/var/run/apache2.pid"
        self.ports_file = "/etc/apache2/ports.conf"
        self.default_site = "/etc/apache2/sites-available/default"
        self.mailman_site = "/etc/apache2/sites-enabled/mailman"
        self.tempdir = tempfile.mkdtemp()

        self.cj = cookielib.LWPCookieJar()
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cj))

        # Make sure daemons are stopped before we begin
        self.postfix_daemon.stop()
        self.mailman_daemon.stop()

        testlib.config_replace(self.mailman_aliases, "", append=True)
        testlib.config_set(self.mailman_cfg,'MTA',"'Postfix'")
        subprocess.call(['/usr/lib/mailman/bin/genaliases'], stdout=subprocess.PIPE)
        subprocess.call(['chown', 'root:list', self.mailman_aliases])
        # Is this a packaging mistake?
        subprocess.call(['chown', 'list:list', '/var/lib/mailman/archives/private'])

        self._zap_lists()
        subprocess.call(['/usr/sbin/newlist', '-q', 'mailman@lists.example.com', 'root@example.com' ,'ubuntu'], stdout=subprocess.PIPE)

        self._setUp_postfix()
        self._setUp_apache()

        self.mailman_daemon.restart()

        self.user = testlib.TestUser(lower=True)
        self.s = None
        # Silently allow for this connection to fail, to handle the
        # initial setup of the postfix server.
        try:
            self.s = smtplib.SMTP('localhost', port=25)
        except:
            pass

    def tearDown(self):
        '''Clean up after each test_* function'''

        try:
            self.s.quit()
        except:
            pass
        self.user = None

        self._zap_lists()

        if os.path.exists(self.tempdir):
            testlib.recursive_rm(self.tempdir)

        testlib.config_restore(self.mailman_cfg)
        testlib.config_restore(self.mailman_aliases)

        self._tearDown_postfix()
        self._tearDown_apache()

        self.mailman_daemon.stop()

    def _zap_lists(self):
        '''Remove existing mailman lists.'''

        if os.path.exists('/var/lib/mailman/lists/testlist'):
            subprocess.call(['/usr/sbin/rmlist', '-a', 'testlist'], stdout=subprocess.PIPE)
        if os.path.exists('/var/lib/mailman/lists/mailman'):
            subprocess.call(['/usr/sbin/rmlist', '-a', 'mailman'], stdout=subprocess.PIPE)

    def _setUp_postfix(self):
        '''Create Postfix server configs.'''
        testlib.config_replace(self.postfix_mastercf, "", append=True)

        testlib.config_set(self.postfix_maincf,'mydestination','example.com, localhost.localdomain, localhost')

        # Move listener to localhost:25
        master = open('/etc/postfix/master.cf.new','w')
        for cfline in open(self.postfix_mastercf):
            if cfline.startswith('smtp') and 'smtpd' in cfline and 'inet' in cfline:
                master.write('127.0.0.1:25      inet  n       -       -       -       -       smtpd\n')
            else:
                master.write(cfline)
        master.write('''mailman   unix  -       n       n       -       -       pipe
  flags=FR user=list argv=/usr/lib/mailman/bin/postfix-to-mailman.py
  ${nexthop} ${user}''')
        master.close()
        os.rename('/etc/postfix/master.cf.new',self.postfix_mastercf)

        # Use mbox only
        testlib.config_comment(self.postfix_maincf,'home_mailbox')
        testlib.config_set(self.postfix_maincf,'mailbox_command','procmail -a "$EXTENSION"')

        # Config mailman
        testlib.config_set(self.postfix_maincf,'relay_domains','lists.example.com')
        testlib.config_set(self.postfix_maincf,'transport_maps','hash:%s' % self.postfix_transport)
        testlib.config_set(self.postfix_maincf,'mailman_destination_recipient_limit','1')
        testlib.config_set(self.postfix_maincf,'alias_maps','hash:%s, hash:%s' % (self.postfix_aliases,self.mailman_aliases))

        testlib.config_replace(self.postfix_transport, "lists.example.com      mailman:")
        subprocess.call(['postmap', self.postfix_transport], stdout=subprocess.PIPE)

        testlib.config_replace(self.postfix_aliases, '''mailman:              "|/var/lib/mailman/mail/mailman post mailman"
mailman-admin:        "|/var/lib/mailman/mail/mailman admin mailman"
mailman-bounces:      "|/var/lib/mailman/mail/mailman bounces mailman"
mailman-confirm:      "|/var/lib/mailman/mail/mailman confirm mailman"
mailman-join:         "|/var/lib/mailman/mail/mailman join mailman"
mailman-leave:        "|/var/lib/mailman/mail/mailman leave mailman"
mailman-owner:        "|/var/lib/mailman/mail/mailman owner mailman"
mailman-request:      "|/var/lib/mailman/mail/mailman request mailman"
mailman-subscribe:    "|/var/lib/mailman/mail/mailman subscribe mailman"
mailman-unsubscribe:  "|/var/lib/mailman/mail/mailman unsubscribe mailman"''', append=True)

        subprocess.call(['chown', 'root:list', self.postfix_aliases])
        subprocess.call(['newaliases'])

        # Restart server
        self.postfix_daemon.restart()
        # Postfix exits its init script before the master listener has started
        time.sleep(2)

    def _tearDown_postfix(self):
        '''Tear down Postfix'''

        self.postfix_daemon.stop()

        testlib.config_restore(self.postfix_mastercf)
        testlib.config_restore(self.postfix_maincf)
        testlib.config_restore(self.postfix_aliases)

        subprocess.call(['chown', 'root:root', self.postfix_aliases])

        if os.path.exists(self.postfix_transport):
            os.unlink(self.postfix_transport)
        if os.path.exists(self.postfix_transport + ".db"):
            os.unlink(self.postfix_transport + ".db")

    def _setUp_apache(self):
        '''Set up Apache'''

        # Change the default port, so we can run in a schroot
        testlib.config_replace(self.ports_file, "", append=True)
        subprocess.call(['sed', '-i', 's/80/8000/g', self.ports_file])
        testlib.config_replace(self.default_site, "", append=True)
        subprocess.call(['sed', '-i', 's/80/8000/g', self.default_site])

        if os.path.exists(self.mailman_site):
            os.unlink(self.mailman_site)

        if self.lsb_release['Release'] == 6.06:
            self._dapper_apache_conf()
        else:
            os.symlink("/etc/mailman/apache.conf", self.mailman_site)

        testlib_httpd.HttpdCommon._setUp(self)

    def _tearDown_apache(self):
        '''Tear down Apache'''

        if os.path.exists(self.mailman_site):
            os.unlink(self.mailman_site)

        testlib.config_restore(self.ports_file)
        testlib.config_restore(self.default_site)
        testlib_httpd.HttpdCommon._tearDown(self)

    def _dapper_apache_conf(self):
        '''Create an example apache conf file for Dapper.'''
        conf_file = open(self.mailman_site,'w')
        conf_file.write('''ScriptAlias /cgi-bin/mailman/ /usr/lib/cgi-bin/mailman/
Alias /pipermail/ /var/lib/mailman/archives/public/
Alias /images/mailman/ /usr/share/images/mailman/
<Directory /usr/lib/cgi-bin/mailman/>
    AllowOverride None
    Options ExecCGI
    AddHandler cgi-script .cgi
    Order allow,deny
    Allow from all
</Directory>
<Directory /var/lib/mailman/archives/public/>
    Options Indexes FollowSymlinks
    AllowOverride None
    Order allow,deny
    Allow from all
</Directory>
<Directory /usr/share/images/mailman/>
    AllowOverride None
    Order allow,deny
    Allow from all
</Directory>
''')
        conf_file.close()

    def _deliver_email(self, from_addr, to_addr, body):
        '''Perform mail delivery'''
        result = self.s.sendmail(from_addr, to_addr, body)

    def _check_email(self, user, pattern, timeout=30):
        '''Get mailman confirmation email'''
        re_pattern = re.compile(pattern)
        spool_file = '/var/mail/%s' % (user.login)
        result = None
        contents = ''
        while timeout > 0:
            if os.path.exists(spool_file):
                contents = open(spool_file).read()
                result = re_pattern.search(contents)
                if result != None:
                    break
            time.sleep(1)
            timeout -= 1
        self.assertTrue(timeout > 0, "Reached timeout searching for pattern in '%s'" % contents)
        return result

    def _get_confirmation(self, user):
        '''Get mailman confirmation email'''
        pattern = 'confirm (\w+)\n'
        result = self._check_email(user, pattern)
        return result.group(1)

    def _test_roundtrip_mail(self, user):
        '''Send and check email delivery'''

        body = '''From: Rooty <root>
To: "%s" <%s@example.com>
Subject: This is test 1

Hello, nice to meet you.
''' % (user.gecos, user.login)

        self._deliver_email('root', user.login + '@example.com', body)
        pattern = "Subject: This is test 1"
        self._check_email(user, pattern)

    def test_aaa_daemons(self):
        '''Test daemon'''

        self.assertTrue(testlib.check_pidfile("apache2", self.apache_pid))
        self.assertTrue(testlib.check_pidfile("python", self.mailman_pid))

    def test_aaa_status(self):
        '''Test status (apache2ctl)'''
        rc, report = testlib.cmd(['apache2ctl', 'status'])
        expected = 0
        result = 'Got exit code %d, expected %d\n' % (rc, expected)
        self.assertEquals(expected, rc, result + report)

    def test_aab_http(self):
        '''Test http'''
        self._test_url("http://localhost:8000/")

        test_str = testlib_httpd.create_html_page(self.html_page)
        self._test_url("http://localhost:8000/" + \
                       os.path.basename(self.html_page), test_str)
        self._test_url("http://localhost:8000/cgi-bin/mailman/listinfo/mailman",
                       "About Mailman")

    def test_aac_sending_mail_direct(self):
        '''Test postfix mail delivery'''
        self._test_roundtrip_mail(self.user)

    def test_baa_mailman_subscribe(self):
        '''Test mailman subscription'''

        password = "Ubuntu"

        # Try and subscribe to the Mailman list
        values = { 'email': self.user.login + "@example.com",
                   'pw': password,
                   'pw-conf': password,
                 }
        data = urllib.urlencode(values)
        req = urllib2.Request('http://localhost:8000/cgi-bin/mailman/subscribe/mailman', data)
        result = self.opener.open(req).read()
        self.assertTrue('Mailman Subscription results' in result, result)

        # Parse the confirmation email
        conf = self._get_confirmation(self.user)

        # Now let's confirm via the web page
        values = { 'cookie': conf,
                   'realname': '',
                   'digests': '0',
                   'language': 'en',
                   'submit': 'Subscribe to list Mailman',
                 }
        data = urllib.urlencode(values)
        req = urllib2.Request('http://localhost:8000/cgi-bin/mailman/confirm/mailman', data)
        result = self.opener.open(req).read()
        self.assertTrue('successfully confirmed your subscription request' in result, result)

        # Send an email to the list
        body = '''From: "%s" <%s@example.com>
To: "Mailman list" <mailman@lists.example.com>
Subject: This is mailman test

Yay! My first post. Ubuntu rocks!
''' % (self.user.gecos, self.user.login)

        self._deliver_email(self.user.login + '@example.com', 'mailman@lists.example.com', body)

        # See if it was delivered
        pattern = "Ubuntu rocks"
        self._check_email(self.user, pattern)

    def test_cve_2010_3089(self):
        '''Test CVE-2010-3089'''

        tempconf = os.path.join(self.tempdir, 'templist-config')

        # Create a test list and insert XSS into description
        subprocess.call(['/usr/sbin/newlist', '-q', 'testlist@lists.example.com', 'root@example.com' ,'ubuntu'], stdout=subprocess.PIPE)
        subprocess.call(['/usr/sbin/config_list', '-o', tempconf, 'testlist'], stdout=subprocess.PIPE)
        testlib.config_set(tempconf,'description',"'<XSSTEST>'")
        subprocess.call(['/usr/sbin/config_list', '-i', tempconf, 'testlist'], stdout=subprocess.PIPE)

        request = "GET /cgi-bin/mailman/listinfo/testlist HTTP/1.1\nHost: localhost\nConnection: close\n\n"
        self._test_raw(request, '<XSSTEST>', port=8000, invert=True)
        self._test_raw(request, '&lt;XSSTEST&gt;', port=8000)

    def test_cve_2011_0707(self):
        '''Test CVE-2011-0707'''

        password = "Ubuntu"

        # Try and subscribe to the Mailman list
        values = { 'email': self.user.login + "@example.com",
                   'realname': '<XSSTEST>',
                   'pw': password,
                   'pw-conf': password,
                 }
        data = urllib.urlencode(values)
        req = urllib2.Request('http://localhost:8000/cgi-bin/mailman/subscribe/mailman', data)
        result = self.opener.open(req).read()
        self.assertTrue('Mailman Subscription results' in result, result)

        # Parse the confirmation email
        conf = self._get_confirmation(self.user)

        # Now let's confirm via the web page
        values = { 'cookie': conf,
                   'realname': '<XSSTEST>',
                   'digests': '0',
                   'language': 'en',
                   'submit': 'Subscribe to list Mailman',
                 }
        data = urllib.urlencode(values)
        req = urllib2.Request('http://localhost:8000/cgi-bin/mailman/confirm/mailman', data)
        result = self.opener.open(req).read()
        self.assertTrue('successfully confirmed your subscription request' in result, result)

        # Kill email so we can parse the new confirmation string
        subprocess.call(['rm','-rf', '/var/mail/'+self.user.login])

        # Look at the unsubscribe page
        values = { 'email': self.user.login + "@example.com",
                   'password': password,
                   'language': 'en',
                   'login-unsub': 'Unsubscribe',
                 }
        data = urllib.urlencode(values)
        req = urllib2.Request('http://localhost:8000/cgi-bin/mailman/options/mailman', data)
        result = self.opener.open(req).read()

        # Parse the confirmation email
        conf = self._get_confirmation(self.user)

        # Now let's confirm via the web page
        values = { 'cookie': conf,
                   'email': self.user.login + "@example.com",
                   'password': password,
                   'language': 'en',
                 }
        data = urllib.urlencode(values)
        req = urllib2.Request('http://localhost:8000/cgi-bin/mailman/confirm/mailman', data)
        result = self.opener.open(req).read()

        self._word_find(result, '<XSSTEST>', invert=True)
        self._word_find(result, '&lt;XSSTEST&gt;')


if __name__ == '__main__':
    # simple
    unittest.main()

    # more configurable
    #suite = unittest.TestSuite()
    #suite.addTest(unittest.TestLoader().loadTestsFromTestCase(PkgTest))
    #rc = unittest.TextTestRunner(verbosity=2).run(suite)
    #if not rc.wasSuccessful():
    #    sys.exit(1)



# -*- python -*-

# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA


"""This is the module which takes your site-specific settings.

From a raw distribution it should be copied to mm_cfg.py.  If you
already have an mm_cfg.py, be careful to add in only the new settings
you want.  The complete set of distributed defaults, with annotation,
are in ./Defaults.  In mm_cfg, override only those you want to
change, after the

  from Defaults import *

line (see below).

Note that these are just default settings - many can be overridden via the
admin and user interfaces on a per-list or per-user basis.

Note also that some of the settings are resolved against the active list
setting by using the value as a format string against the
list-instance-object's dictionary - see the distributed value of
DEFAULT_MSG_FOOTER for an example."""


#######################################################
#    Here's where we get the distributed defaults.    #

from Defaults import *

##############################################################
# Put YOUR site-specific configuration below, in mm_cfg.py . #
# See Defaults.py for explanations of the values.            #

#-------------------------------------------------------------
# The name of the list Mailman uses to send password reminders
# and similar. Don't change if you want mailman-owner to be
# a valid local part.
MAILMAN_SITE_LIST = 'mailman'

#-------------------------------------------------------------
# If you change these, you have to configure your http server
# accordingly (Alias and ScriptAlias directives in most httpds)
DEFAULT_URL_PATTERN = 'http://%s/cgi-bin/mailman/'
IMAGE_LOGOS         = '/images/mailman/'

#-------------------------------------------------------------
# Default domain for email addresses of newly created MLs
DEFAULT_EMAIL_HOST = 'thunderchild.aszi.sztaki.hu'
#-------------------------------------------------------------
# Default host for web interface of newly created MLs
DEFAULT_URL_HOST   = 'thunderchild.aszi.sztaki.hu'
#-------------------------------------------------------------
# Required when setting any of its arguments.
add_virtualhost(DEFAULT_URL_HOST, DEFAULT_EMAIL_HOST)

#-------------------------------------------------------------
# The default language for this server.
DEFAULT_SERVER_LANGUAGE = 'en'

#-------------------------------------------------------------
# Iirc this was used in pre 2.1, leave it for now
USE_ENVELOPE_SENDER    = 0              # Still used?

#-------------------------------------------------------------
# Unset send_reminders on newly created lists
DEFAULT_SEND_REMINDERS = 0

#-------------------------------------------------------------
# Uncomment this if you configured your MTA such that it
# automatically recognizes newly created lists.
# (see /usr/share/doc/mailman/README.Exim4.Debian or
# /usr/share/mailman/postfix-to-mailman.py)
# MTA=None   # Misnomer, suppresses alias output on newlist

#-------------------------------------------------------------
# Uncomment if you use Postfix virtual domains (but not
# postfix-to-mailman.py), but be sure to see
# /usr/share/doc/mailman/README.Debian first.
# MTA='Postfix'

#-------------------------------------------------------------
# Uncomment if you want to filter mail with SpamAssassin. For
# more information please visit this website:
# http://www.jamesh.id.au/articles/mailman-spamassassin/
# GLOBAL_PIPELINE.insert(1, 'SpamAssassin')

# Note - if you're looking for something that is imported from mm_cfg, but you
# didn't find it above, it's probably in /usr/lib/mailman/Mailman/Defaults.py.



# Copyright (C) 2002-2003 by James Henstridge <james@daa.com.au>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, US

'''Module used to communicate with a SpamAssassin spamd process to check
or tag messages.

Usage is as follows:
  >>> conn = spamd.SpamdConnection()
  >>> conn.addheader('User', 'username')
  >>> conn.check(spamd.SYMBOLS, 'From: user@example.com\n...')
  >>> print conn.getspamstatus()
  (True, 4.0)
  >>> print conn.response_message
  ...
'''

import socket
import mimetools, StringIO

import __builtin__
if not hasattr(__builtin__, 'True'):
    __builtin__.True = (1 == 1)
    __builtin__.False = (1 != 1)
del __builtin__

class error(Exception): pass

SPAMD_PORT = 783

# available methods
SKIP          = 'SKIP'
PROCESS       = 'PROCESS'
CHECK         = 'CHECK'
SYMBOLS       = 'SYMBOLS'
REPORT        = 'REPORT'
REPORT_IFSPAM = 'REPORT_IFSPAM'

# error codes
EX_OK          = 0
EX_USAGE       = 64
EX_DATAERR     = 65
EX_NOINPUT     = 66
EX_NOUSER      = 67
EX_NOHOST      = 68
EX_UNAVAILABLE = 69
EX_SOFTWARE    = 70
EX_OSERR       = 71
EX_OSFILE      = 72
EX_CANTCREAT   = 73
EX_IOERR       = 74
EX_TEMPFAIL    = 75
EX_PROTOCOL    = 76
EX_NOPERM      = 77
EX_CONFIG      = 78

class SpamdConnection:
    '''Class to handle talking to SpamAssassin spamd servers.'''
    # default spamd 
    host = 'localhost'
    port = SPAMD_PORT

    PROTOCOL_VERSION = 'SPAMC/1.3'

    def __init__(self, host='', port=0):
        if not port and ':' in host:
            host, port = host.split(':', 1)
            port = int(port)
        if host: self.host = host
        if port: self.port = port

        # message structure to hold request headers
        self.request_headers = mimetools.Message(StringIO.StringIO(), seekable=False)
        self.request_headers.fp = None

        # stuff that will be filled in after check()
        self.server_version = None
        self.result_code = None
        self.response_message = None
        self.response_headers = mimetools.Message(StringIO.StringIO(), seekable=False)

    def addheader(self, header, value):
        '''Adds a header to the request.'''
        self.request_headers[header] = value

    def check(self, method='PROCESS', message=''):
        '''Sends a request to the spamd process.'''
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect((self.host, self.port))
        except socket.error:
            raise error('could not connect to spamd on %s' % self.host)

        # set content length request header
        del self.request_headers['Content-length']
        self.request_headers['Content-length'] = str(len(message))

        request = '%s %s\r\n%s\r\n' % \
                  (method, self.PROTOCOL_VERSION,
                   str(self.request_headers).replace('\n', '\r\n'))

        try:
            sock.send(request)
            sock.send(message)
            sock.shutdown(1) # shut down the send half of the socket
        except (socket.error, IOError):
            raise error('could not send request to spamd')

        fp = sock.makefile('rb')
        response = fp.readline()
        words = response.split(None, 2)
        if len(words) != 3:
            raise error('not enough words in response header')
        if words[0][:6] != 'SPAMD/':
            raise error('bad protocol name in response string')
        self.server_version = float(words[0][6:])
        if self.server_version < 1.0 or self.server_version >= 2.0:
            raise error('incompatible server version')
        self.result_code = int(words[1])
        if self.result_code != 0:
            raise error('spamd server returned error %s' % words[2])

        try:
            # parse header
            self.response_headers = mimetools.Message(fp, seekable=False)
            self.response_headers.fp = None
        except IOError:
            raise error('could not read in response headers')

        try:
            # read in response message
            self.response_message = fp.read()
        except IOError:
            raise error('could not read in response message')
            
        fp.close()
        sock.close()

    def getspamstatus(self):
        '''Decode the "Spam" response header.'''
        if not self.response_headers.has_key('Spam'):
            raise error('Spam header not found in response')

        isspam, score = self.response_headers['Spam'].split(';', 1)
        isspam = (isspam.strip() != 'False')
        score = float(score.split('/',1)[0])
        return isspam, score



#! /usr/bin/env python

# $URL: svn+bsb://svn.winnegan.de/svn/adm/trunk/mailman/postfix-to-mailman.py $
# $Id: postfix-to-mailman.py 38 2004-04-15 19:11:32Z bsb $
#
# Interface mailman to a postfix with a mailman transport. Does not require
# the creation of _any_ aliases to connect lists to your mail system.
#
# Dax Kelson, dkelson@gurulabs.com, Sept 2002.
# coverted from qmail to postfix interface
# Jan 2003: Fixes for Mailman 2.1
# Thanks to Simen E. Sandberg <senilix@gallerbyen.net>
# Feb 2003: Change the suggested postfix transport to support VERP
# Thanks to Henrique de Moraes Holschuh <henrique.holschuh@ima.sp.gov.br>
#
# Mar 2004: Siggy Brentrup <bsb@debian.org>
#   downloaded from http://www.gurulabs.com/files/postfix-to-mailman-2.1.py
#   and adopted for inclusion in the Debian Mailman package.
#   (hi Bruce, back to the roots :-)
#   rewritten for python >= 2.2 taking configuration from mm_cfg
#
# This script was originally qmail-to-mailman.py by:
# Bruce Perens, bruce@perens.com, March 1999.
# This is free software under the GNU General Public License.

# This script is meant to be called as a postfix transport pipe.

# It catches all mail to a virtual domain, eg "lists.example.com".  It
# looks at the recipient for each mail message and decides if the mail
# is addressed to a valid list or not, and optionally bounces the
# message with a helpful suggestion if it's not addressed to a
# list. It decides if it is a posting, a list command, or mail to the
# list administrator, by checking for the -admin, -owner, -request,
# -join, -leave, -subscribe and -unsubscribe addresses. It will
# recognize a list as soon as the list is created, there is no need to
# add _any_ aliases for any list.  It recognizes mail to postmaster,
# abuse and mailer-daemon, and routes those mails to DEB_LISTMASTER as
# defined in mm_cfg.py

# INSTALLATION:
#
# Install this file as /var/lib/mailman/bin/postfix-to-mailman.py
#
# To configure a virtual domain to connect to mailman, edit Postfix thusly:
#
# /etc/postfix/main.cf:
#    relay_domains = ... lists.example.com
#    relay_recipient_maps = ... hash:/var/lib/mailman/data/virtual-mailman
#    transport_maps = hash:/etc/postfix/transport
#    mailman_destination_recipient_limit = 1
#
# /etc/postfix/master.cf
#    mailman unix  -       n       n       -       -       pipe
#      flags=FR user=list 
#      argv=/var/lib/mailman/bin/postfix-to-mailman.py ${nexthop} ${user}
#
# /etc/postfix/transport:
#   lists.example.com   mailman:
#
# /etc/mailman/mm_cfg.py
#    MTA = None # So that mailman skips aliases generation
#    POSTFIX_STYLE_VIRTUAL_DOMAINS = ['lists.example.com']
#    # alias for postmaster, abuse and mailer-daemon
#    DEB_LISTMASTER = 'postmaster@example.com'
#
# Replace lists.example.com above with the name of the domain to be
# connected to Mailman. Note that _all_ mail to that domain will go to
# Mailman, so you don't want to put the name of your main domain
# here. Typically a virtual domain lists.domain.com is used for
# Mailman, and domain.com for regular email.
#
# The recipient map allows Postfix to know which addresses exists.
# Thus, if someone tries to send a (spam?) message to an undefined
# address in the domain connected to Mailman, Postfix will just refuse
# it instead of sending a (backscatter?) bounce.
#
# When you are done, restart Postfix, and run /usr/lib/mailman/bin/genaliases
# to generate the initial recipient map for the existing mailing-lists.

# Exit codes accepted by postfix
#  from postfix-2.0.16/src/global/sys_exits.h
EX_USAGE    = 64    # command line usage error 
EX_NOUSER   = 67    # addressee unknown
EX_SOFTWARE = 70    # internal software error
EX_TEMPFAIL = 75    # temporary failure

import sys, os
sys.path.append("/usr/lib/mailman/bin")
import paths

from Mailman import mm_cfg

def main():
    os.nice(5)     # Handle mailing lists at non-interactive priority.
                   # delete this if you wish

    try:
        MailmanOwner = mm_cfg.DEB_LISTMASTER
    except AttributeError:
        MailmanOwner = 'postmaster@localhost'

    try:
        domain, local = [ a.lower() for a in sys.argv[1:] ]
    except:
        # This might happen if we're not using Postfix or
        # /etc/postfix/master.cf is badly misconfigured
        sys.stderr.write('Illegal invocation: %r\n'
                         % ' '.join(sys.argv))
        if len(sys.argv) > 3:
            sys.stderr.write('Did you forget to set '
                             'mailman_destination_recipient_limit=1 '
                             'in main.cf?')
        sys.exit(EX_USAGE)

    # Redirect required addresses to 
    if local in ('postmaster', 'abuse', 'mailer-daemon'):
        os.execv("/usr/sbin/sendmail",
                 ("/usr/sbin/sendmail", MailmanOwner))
        sys.exit(0)

    # Assume normal posting to a mailing list
    mlist, func = local, 'post'

    # Let Mailman decide if a list exists.
    from Mailman.Utils import list_exists

    if list_exists(mlist):
        mm_pgm = os.path.join(paths.prefix, 'mail', 'mailman')
        os.execv(mm_pgm, (mm_pgm, func, mlist))
        # NOT REACHED

    # Check for control extension on local part
    for ext in ('-admin',
                '-owner',
                '-request',
                '-bounces',
                '-confirm',
                '-join',
                '-leave',
                '-subscribe',
                '-unsubscribe',
                ):
        if local.endswith(ext):
            mlist = local[:-len(ext)]
            func  = ext[1:]
            break

    if list_exists(mlist):
        mm_pgm = os.path.join(paths.prefix, 'mail', 'mailman')
        os.execv(mm_pgm, (mm_pgm, func, mlist))
        # NOT REACHED
    else:
        try:
            sys.stderr.write(mm_cfg.DEB_HELP_TEXT)
        except AttributeError:
            sys.exit(EX_NOUSER)

        sys.exit(1)


if __name__ == '__main__':
    try:
        main()
    except SystemExit, argument:
        sys.exit(argument)
    except Exception:
        xt, xv, tb = sys.exc_info()
        sys.stderr.write("%s %s\n" % (xt, xv))
        sys.stderr.write("Line %d\n" % (tb.tb_lineno))
        sys.exit(EX_TEMPFAIL) # Soft failure, try again later.
